<tr xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://genshi.edgewall.org/"
    class="${css_class}"
    py:attrs="attrs">
    <td>
        <b>$identifier.name</b>
    </td>
    <td> ${value[identifier.name]}
    </td>
</tr>