<html xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude">
<py:def function="header">
  <div id="header">
    <div class="alogo">
      <img src="../images/tg2_04.gif" alt="TG2!" height="40" width="84" />
    </div>
    <div class="headtext">
     <span py:replace="page"/> - 
    change me in header.html
    </div>
  </div>
</py:def>    
</html>