<html xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude">
<py:def function="footer">
<div id="footer"> 
  	<div class="flogo">
  	  <img src="images/under_the_hood_blue.png" alt="TurboGears" />
  	</div>
  	<div class="foottext">
  	  <p>TurboGears 2 is a open source front-to-back web development
      framework written in Python. Copyright (c) 2005-<span py:replace="now.strftime('%Y')">current year</span> </p>
    </div>
  </div>
</py:def>    
</html>