<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude"
      py:strip="">
<xi:include href="sitetemplate.html" />
<head py:match="head" py:attrs="select('@*')">
    <meta content="text/html; charset=UTF-8" http-equiv="content-type" py:replace="''"/>
    <title py:replace="''">Your title goes here</title>
        <link py:for="css in tg_css" py:replace="css.display()" />
    <link py:for="js in tg_js_head" py:replace="js.display()" />
    <link py:replace="tg.dbmech_css()" />
    <meta py:replace="select('*')"/>
    <!--<link rel="stylesheet" type="text/css" media="screen" href="/static/css/style.css" />-->
</head>

<body py:match="body" py:attrs="select('@*')">
  <div id="header">&nbsp;</div>
  <div id="main_content">
  <div id="status_block" py:if="defined('tg_flash') and tg_flash" class="flash" py:content="tg_flash"></div>
  <div py:for="js in tg_js_bodytop" py:replace="js.display()" />
  <div py:replace="select('*|text()')"/>
  <div py:for="js in tg_js_bodybottom" py:replace="js.display()" />
  <!-- End of main_content -->
  </div>
  <div id="footer"> 
<!--    <img src="/static/images/under_the_hood_blue.png" alt="TurboGears" />-->
    ${tg.dbmech_footer_img()}
    <img src="/toscawidgets/resources/dbsprockets/dbmechanic/static/images/tg_under_the_hood.png"/>
  <p>TurboGears is a open source front-to-back web development
    framework written in Python</p>
    <p>Copyright (c) 2005-2007 </p>
  </div>
</body>

</html>

