<html xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude">

<py:def function="sidebar_top">
  <div id="sb_top" class="sidebar">
      <h2>Get Started with TG2</h2>
      <span class="white">Read, mod, run</span>
      <ul class="links">
        <li py:choose="">
          <span py:when="page=='index'"><a href="/about">About this page</a> A quick guide to this TG2 site </span>
          <span py:otherwise=""><a href="/">Home</a> Back to your Quickstart Home page </span>
        </li>
        <li><a href="http://docs.turbogears.org/2.0/RoughDocs/">TG2 Documents</a> - Read everything in the Getting Started section</li>
        <li><a href="http://docs.turbogears.org/1.0">TG1 docs</a> (still useful, although a lot has changed for TG2) </li>
        <li><a href="http://groups.google.com/group/turbogears"> Join the TG Mail List</a> for general TG use/topics  </li>
      </ul>
  </div>
</py:def>    

<py:def function="sidebar_bottom">
  <div id="sb_bottom" class="sidebar">
      <h2>Developing TG2</h2>
      <span class="white">Jump in and Contribute</span>
      <ul class="links">
        <li><a href="http://docs.turbogears.org/2.0/RoughDocs/">TG2 Documents</a> A work in progress</li>
	    <li><a href="http://trac.turbogears.org/query?milestone=2.0&amp;order=priority">TG2 Trac tickets</a> What's happening now in TG2 dev</li>
        <li><a href="http://trac.turbogears.org/timeline">TG Dev timeline</a> (recent ticket updates, svn checkins, wiki changes)</li>
        <li><a href="http://trac.turbogears.org/browser">TG2 Trac repository</a> TG2 is in /trunk</li>
        <li><a href="http://groups.google.com/group/turbogears-trunk"> Join the TG-Trunk Mail List</a> just for TG2 discuss/dev </li>
      </ul>
  </div>
</py:def>    

</html>
