<span xmlns="http://www.w3.org/1999/xhtml"
       xmlns:py="http://genshi.edgewall.org/"
       >
<input type="checkbox" name="${name}" class="${css_class}" id="${id}"
       value="True"
       py:attrs="attrs" checked="${value}" />
<input type="hidden" name="${name}" id="$id" value="False" />
</span>