<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude"
      py:strip="">
<xi:include href="tw_resources.html" />
<?python
import tg
tg_flash = tg.get_flash()
?>
<head py:match="head" py:attrs="select('@*')">
    <meta content="text/html; charset=UTF-8" http-equiv="content-type" py:replace="''"/>
    <title py:replace="''">Your title goes here</title>
	<link py:for="css in tg_css" py:replace="ET(css.display())" />
<!--    <link py:replace="tmpl_context.css()" />    	-->
    <link py:for="js in tg_js_head" py:replace="ET(js.display())" />
    <meta py:replace="select('*')"/>
    <link rel="stylesheet" type="text/css" media="screen" href="/css/style.css" />
</head>

<body py:match="body" py:attrs="select('@*')">
  <div id="header">&nbsp;</div>
  <div id="main_content">
  <div id="status_block" py:if="defined('tg_flash') and tg_flash" class="flash" py:content="tg_flash"></div>
  <div py:for="js in tg_js_bodytop" py:replace="ET(js.display())" />
  <div py:replace="select('*|text()')"/>
  <div py:for="js in tg_js_bodybottom" py:replace="ET(js.display())" />
  <!-- End of main_content -->
  </div>
  <div id="footer">
       <p>TurboGears 2 is a open source front-to-back web development framework written in Python</p>
       <p>Copyright (c) 2005-2008 </p>
  </div>
</body>
</html>
