<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude"
      py:strip="">
<?python
import tg
tg_flash = tg.get_flash()
tg_status = tg.get_status()
?>
<xi:include href="${tg.url('tw_resources.html')}" />
<head py:match="head" py:attrs="select('@*')">
  <meta content="text/html; charset=UTF-8" http-equiv="content-type" py:replace="''"/>
  <title py:replace="''">Your title goes here</title>
  <meta py:replace="select('*')"/>
  <link rel="stylesheet" type="text/css" media="screen" href="${tg.url('/css/style.css')}" />
</head>

<body py:match="body" py:attrs="select('@*')">
  <h1 id="header" title="TurboGears 2.0">&nbsp;</h1>

  <div id="main_content">

    <div id="${tg_status}" py:if="tg_flash" class="flash" py:content="tg_flash"
    ></div>
    <!-- End of status_block -->

    <!-- Start of page template body -->
    <div py:replace="select('*|text()')"/>
    <!-- End of page template body -->
  </div>
  <!-- End of main_content -->

  <div id="footer"> 
    <img src="images/under_the_hood_blue.png" alt="TurboGears" />
    <p>TurboGears 2 is an open source front-to-back web development
    framework written in Python</p>

    <p>Copyright &#xa9; 2005-2008</p>
  </div>
  <!-- End of footer -->
</body>

</html>
