<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude">

<xi:include href="master.html" />

<head>
  <meta content="text/html; charset=UTF-8" http-equiv="content-type" py:replace="''"/>
  <title>Welcome to TurboGears 2.0 - standing on the shoulders of giants 
    since 2007</title>
</head>

<body>
    <!-- Debug template -->
    <!-- 
    <h1>All objects from locals():<h1>

    <div py:for="item in sorted(locals()['data'].keys())">
      ${item}: ${repr(locals()['data'][item])}</div>
    -->

    <div id="sidebar">
      <h2>Getting help</h2>

      <ul class="links">
        <li><a href="http://docs.turbogears.org/2.0">Documentation</a></li>
        <li><a href="http://docs.turbogears.org/2.0/API">API Reference</a></li>
        <li><a href="http://trac.turbogears.org/turbogears/">Bug Tracker</a></li>
        <li><a href="http://groups.google.com/group/turbogears">Mailing 
        List</a></li>
      </ul>

      <p><span py:replace="now.strftime('%Y-%m-%d %H:%M')">now</span></p>
    </div>
    <!-- End of sidebar -->

    <div id="getting_started">
      <ol id="getting_started_steps">
        <li class="getting_started">
          <h2>Design your data model</h2>

          <p>Design your data model, create the database and add some 
          bootstrap data.</p>
        </li>

        <li class="getting_started">
          <h2>Style your URLs</h2>

          <p> Style your URLs, program your controller methods, write your 
          templates and add some static files (CSS and/or JavaScript).</p>
        </li>
      
        <li class="getting_started">
          <h2>Build a distribution</h2>

          <p> Build a distribution, test your source, generate project 
          documents.</p>
        </li>
      </ol>

      <div class="notice"> TurboGears is an open source project.
        <a href="http://docs.turbogears.org/Contributing">Contribute</a> 
        to make TurboGears even better and your life more exciting.
      </div>
    </div>
    <!-- End of getting_started -->
</body>
</html>
