"""
View Module

Classes:
Name                               Description
View

Exceptions:
None

Functions:
None

Copywrite (c) 2007 Christopher Perkins
Original Version by Christopher Perkins 2007
Released under MIT license.
"""
class View:
    def __init__(self, widget, viewConfig):
        self.widget = widget
        self.viewConfig = viewConfig